# -*- coding: utf-8 -*-
"""
Created on Fri Aug 26 09:32:50 2022

@author: RobWen
Version: 0.4.0
"""

# Packages
import requests
from bs4 import BeautifulSoup

# Header
from .TickerRequest import *

class GuruFocusRequest(TickerRequest):
    def __init__(self, ticker, headers_standard):
        super().__init__(ticker, headers_standard)
        self.__headers_standard = headers_standard

    ##############################
    ###                        ###
    ###  Gurufocus Requests    ###
    ###                        ###
    ##############################

    @property
    def pe_ratio_av(self):
        return self.__gurufocus_pe_ratio_av_v()

    @property
    def debt_to_ebitda(self):
        return self.__gurufocus_debt_to_ebitda()
       
    ##########################
    ###                    ###
    ###  Gurufocus Data    ###
    ###                    ###
    ##########################
    
    def __stock_exchange(self):
        r = requests.get(f'https://www.gurufocus.com/stock/{self.ticker}/summary')
        stock_exchange = BeautifulSoup(r.content, 'html.parser')
        
        try:
            stock_exchange = stock_exchange.find('span', {'class':'t-label'}).text.split()[0]
        except:
            return None
        
        return stock_exchange
    
    def __gurufocus_pe_ratio_av_v(self):
        if self.__stock_exchange() != None:
            r = requests.get(f'https://www.gurufocus.com/term/pettm/{self.__stock_exchange()}/PE-Ratio-TTM/')
            page = BeautifulSoup(r.content, 'html.parser')
        
            table = page.find('div', {'class':'history_bar value'})
            try:
                table = table.find('strong').text.split()
                self.__PE_Ratio_Average = float(table[3])
                return self.__PE_Ratio_Average
            except:
                return None
        else:
            return None
        
    def __gurufocus_debt_to_ebitda(self):
        
        if self.__stock_exchange() != None:
          
            url = f'https://www.gurufocus.com/term/debt2ebitda/{self.__stock_exchange()}/Debt-to-EBITDA'
            page = requests.get(url)
            page = BeautifulSoup(page.content, 'html.parser')

            table = page.find('div', {'class':'history_bar value'})

            try:
                table = table.find('strong')
                table = table.text.split()
                debt_to_EBITDA = table[7]
                try:
                    self.__debt_to_EBITDA = float(debt_to_EBITDA)
                except:
                    return '#'
                return self.__debt_to_EBITDA
            except (AttributeError):
                return '#'


#  ______     __                          __        __    __                               
# /      \   |  \                        |  \      |  \  |  \                              
#|  $$$$$$\ _| $$_     ______    _______ | $$   __ | $$  | $$  ______    ______    ______  
#| $$___\$$|   $$ \   /      \  /       \| $$  /  \| $$__| $$ /      \  /      \  /      \ 
# \$$    \  \$$$$$$  |  $$$$$$\|  $$$$$$$| $$_/  $$| $$    $$|  $$$$$$\|  $$$$$$\|  $$$$$$\
# _\$$$$$$\  | $$ __ | $$  | $$| $$      | $$   $$ | $$$$$$$$| $$    $$| $$   \$$| $$  | $$
#|  \__| $$  | $$|  \| $$__/ $$| $$_____ | $$$$$$\ | $$  | $$| $$$$$$$$| $$      | $$__/ $$
# \$$    $$   \$$  $$ \$$    $$ \$$     \| $$  \$$\| $$  | $$ \$$     \| $$       \$$    $$
#  \$$$$$$     \$$$$   \$$$$$$   \$$$$$$$ \$$   \$$ \$$   \$$  \$$$$$$$ \$$        \$$$$$$ 

__title__ = 'StockHero'
__description__ = 'stock market data downloader'
__url__ = 'https://github.com/RobWen/StockHero'
__version__ = '0.4.0'
__author__ = 'RobWen'
__author_email__ = 'RobWen.StockHero@gmail.com'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2022'
__thanks__ = 'Thanks to my colleagues @Fraunhofer IIS'

'''
Release History

0.4.0 (26.08.2022)
    - introducing a new architecture (again ^^)

0.3.3 (06.07.2022)
    - Some Morningstar Features are up and running again
    - Morningstar is now also working with ISIN

0.3.2 (27.06.2022)
    - Yahoo Finance now able to handle ISIN

0.3.1 (25.06.2022)
    - introducing a new architecture

0.2.10 (xx.06.2022)
    - minor fixes (Morningstar Quote)

0.2.9 (15.06.2022)
    - minor fixes (Fear and Greed Index)

0.2.8 (05.06.2022)
    - minor fixes

0.2.7 (09.12.2021)
    - added some indizies (Dax, MDax, Euro Stoxx 50, Dow Jones, Nasdaq etc.) from Börse Hamburg / Hannover
    - added Debt-to-EBITDA from Gurufocus
    - fixed some bugs

0.2.6 (22.11.2021)
    - added the Fear and Greed Index from CNN
    - added Historical PE-Ratio from Gurufocus
    - optimised NASDAQ summary, looks much nicer now
    - optimised Morningstar Quote, now also NYSE listed stocks work

0.2.5 (29.10.2021)
    - combined nasdaq_pe_forecast / nasdaq_gr_forecast / nvda.nasdaq_peg_forecast (MultiIndex)
    - added yahoo_statistics_p (a bit of PreProcessing)
    - code cleanup
    - np.NaN for any not valid value
    - None for any empty pd.dataframe
    
'''


# -*- coding: utf-8 -*-
"""
Created on Fri Jul 22 09:18:54 2022

@author: RobWen
Version: 0.4.0
"""

# Packages
import pandas as pd
import requests
import numpy as np
from bs4 import BeautifulSoup

# Header
from .TickerRequest import *

class YahooRequest(TickerRequest):
    def __init__(self, ticker, headers_standard):
        super().__init__(ticker, headers_standard)
        self.__headers_standard = headers_standard

    ###########################
    ###                     ###
    ###     Yahoo Finance   ###
    ###        Requests     ###
    ###                     ###
    ###########################

    @property
    def statistics(self):
        return self.__yahoo_statistics_abfrage()

    @property
    def statistics_p(self):
        return self.__yahoo_statistics_df_p()

    #######################
    ###                 ###
    ###  Yahoo Finance  ###
    ###      Data       ###
    ###                 ###
    #######################

    # Führt eine Abfrage durch um das Symbol zu finden
    def __yahoo_statistics_abfrage(self):

        if self.__yahoo_statistics_df_() is None:
            if self.__yahoo_query_df() != None:
                self.ticker = self.__yahoo_query_df()

        return self.__yahoo_statistics_df_()

    def __yahoo_query_df(self):
        url = "https://query2.finance.yahoo.com/v1/finance/search"
        params = {'q': f'{self.ticker}', 'quotesCount': 1, 'newsCount': 0}

        r = requests.get(url, params=params, headers = self.__headers_standard)

        try:
            symbol = r.json()['quotes'][0]['symbol']
        except:
            return None

        return symbol

    ### Yahoo Finance Statistics                                 ###
    ### e.g. https://finance.yahoo.com/quote/NVDA/key-statistics ###
    ### Rückgabe None implementiert und getestet                 ###
    ### Ungültige Werte = NaN implementiert                      ###
    def __yahoo_statistics_df_(self):
        url = f'https://finance.yahoo.com/quote/{self.ticker}/key-statistics'

        with requests.session():
            page = requests.get(url, headers = self.__headers_standard)
            page = BeautifulSoup(page.content, 'html.parser')
            table = page.find_all('td', {'class': 'Fw(500) Ta(end) Pstart(10px) Miw(60px)'})

            if len(table) == 0:
                self.__yahoo_statistics_df = None
            else:
                headlines = page.find_all('h3', {'class': 'Mt(20px)'})

                valuation_measures = []
                n = 9
                v = page.find_all('h2', {'class': 'Pt(20px)'})[0].text
                valuation_measures += n * [v]

                stock_price_history = []
                n = 7
                v = headlines[0].text
                stock_price_history += n * [v]

                share_statistics = []
                n = 12
                v = headlines[1].text
                share_statistics += n * [v]

                dividends_splits = []
                n = 10
                v = headlines[2].text
                dividends_splits += n * [v]

                fiscal_year = []
                n = 2
                v = headlines[3].text
                fiscal_year += n * [v]

                profitability = []
                n = 2
                v = headlines[4].text
                profitability += n * [v]

                management_effectiveness = []
                n = 2
                v = headlines[5].text
                management_effectiveness += n * [v]

                income_statement = []
                n = 8
                v = headlines[6].text
                income_statement += n * [v]

                balance_sheet = []
                n = 6
                v = headlines[7].text
                balance_sheet += n * [v]

                cash_flow_statement = []
                n = 2
                v = headlines[8].text
                cash_flow_statement += n * [v]

                outer_text = page.find_all('td', {
                    'class': 'Pos(st) Start(0) Bgc($lv2BgColor) fi-row:h_Bgc($hoverBgColor) Pend(10px) Miw(140px)'})
                inner_text = page.find_all('td', {
                    'class': 'Pos(st) Start(0) Bgc($lv2BgColor) fi-row:h_Bgc($hoverBgColor) Pend(10px)'})

                if len(outer_text) == 10 or len(inner_text) == 50:

                    arrays = [
                        np.array(valuation_measures + stock_price_history + share_statistics + dividends_splits
                               + fiscal_year + profitability + management_effectiveness + income_statement
                               + balance_sheet + cash_flow_statement),

                        np.array([outer_text[0].text[:-2],
                                  inner_text[0].text[:-2],
                                  inner_text[1].text[:-1],
                                  inner_text[2].text[:-2],
                                  inner_text[3].text[:-2],
                                  inner_text[4].text,
                                  inner_text[5].text,
                                  inner_text[6].text[:-2],
                                  inner_text[7].text[:-2],
                                  outer_text[1].text.strip(),
                                  inner_text[8].text[:-2],
                                  inner_text[9].text[:-2],
                                  inner_text[10].text[:-2],
                                  inner_text[11].text[:-2],
                                  inner_text[12].text[:-2],
                                  inner_text[13].text[:-2],
                                  outer_text[2].text[:-2],
                                  inner_text[14].text[:-2],
                                  inner_text[15].text[:-2],
                                  inner_text[16].text[:-2],
                                  inner_text[17].text[:-2],
                                  inner_text[18].text[:-2],
                                  inner_text[19].text[:-2],
                                  inner_text[20].text[:-2],
                                  inner_text[21].text[:-2],
                                  inner_text[22].text[:-2],
                                  inner_text[23].text[:-2],
                                  inner_text[24].text[:-2],
                                  outer_text[3].text[:-2],
                                  inner_text[25].text[:-2],
                                  inner_text[26].text[:-2],
                                  inner_text[27].text[:-2],
                                  inner_text[28].text[:-2],
                                  inner_text[29].text[:-2],
                                  inner_text[30].text[:-2],
                                  inner_text[31].text[:-2],
                                  inner_text[32].text[:-2],
                                  inner_text[33].text[:-2],
                                  outer_text[4].text[:-1],
                                  inner_text[34].text,
                                  outer_text[5].text[:-1],
                                  inner_text[35].text,
                                  outer_text[6].text,
                                  inner_text[36].text,
                                  outer_text[7].text,
                                  inner_text[37].text,
                                  inner_text[38].text,
                                  inner_text[39].text,
                                  inner_text[40].text.strip(),
                                  inner_text[41].text,
                                  inner_text[42].text,
                                  inner_text[43].text,
                                  outer_text[8].text,
                                  inner_text[44].text,
                                  inner_text[45].text,
                                  inner_text[46].text,
                                  inner_text[47].text,
                                  inner_text[48].text,
                                  outer_text[9].text,
                                  inner_text[49].text]), ]

                    array_table = []

                    for i in range(0, 60):
                        array_table.append(table[i].text.strip())

                    s = pd.DataFrame(array_table, index=arrays, columns=[self.ticker + ' Yahoo Statistics'])
                    s = s.loc[:, self.ticker + ' Yahoo Statistics'].replace(['N/A'], np.nan)

                    self.__yahoo_statistics_df = s.to_frame(name=self.ticker + ' Yahoo Statistics')

                else:
                    self.__yahoo_statistics_df = None

        return self.__yahoo_statistics_df

    ### Yahoo Finance Statistics - PreProcessing                 ###
    ### e.g. https://finance.yahoo.com/quote/NVDA/key-statistics ###
    ### Rückgabe None implementiert und getestet                 ###
    def __yahoo_statistics_df_p(self):

        def m_b_t(string):

            if type(string) != float:
                if string[-1] == 'B':
                    string = float(string[:-1]) * 10 ** 9
                elif string[-1] == 'M':
                    string = float(string[:-1]) * 10 ** 6
                elif string[-1] == 'T':
                    string = float(string[:-1]) * 10 ** 12
                else:
                    string = float(string[:-1])
            else:
                pass

            return string

        s = self.__yahoo_statistics_abfrage()

        if s is not None:
            s.iloc[0, 0] = m_b_t(s.iloc[0, 0])  # Market Cap
            s.iloc[1, 0] = m_b_t(s.iloc[1, 0])  # Enterprise Value
            s.iloc[2, 0] = float(s.iloc[2, 0])  # Trailing P/E
            s.iloc[3, 0] = float(s.iloc[3, 0])  # Forward P/E
            s.iloc[4, 0] = float(s.iloc[4, 0])  # PEG Ratio (5 yr expected)
            s.iloc[5, 0] = float(s.iloc[5, 0])  # Price/Sales (ttm)
            s.iloc[6, 0] = float(s.iloc[6, 0])  # Price/Book (mrq)
            s.iloc[7, 0] = float(s.iloc[7, 0])  # Enterprise Value/Revenue
            s.iloc[8, 0] = float(s.iloc[8, 0])  # Enterprise Value/EBITDA
            s.iloc[9, 0] = float(s.iloc[9, 0])  # Beta (5Y Monthly)
            s.iloc[10, 0] = m_b_t(s.iloc[10, 0])  # 52-Week Change
            s.iloc[11, 0] = m_b_t(s.iloc[11, 0])  # S&P500 52-Week Change
            s.iloc[12, 0] = float(s.iloc[12, 0])  # 52 Week High
            s.iloc[13, 0] = float(s.iloc[13, 0])  # 52 Week Low
            s.iloc[14, 0] = float(s.iloc[14, 0])  # 50-Day Moving Average
            s.iloc[15, 0] = float(s.iloc[15, 0])  # 200-Day Moving Average
            s.iloc[16, 0] = m_b_t(s.iloc[16, 0])  # Avg Vol (3 month)
            s.iloc[17, 0] = m_b_t(s.iloc[17, 0])  # Avg Vol (10 day)
            s.iloc[18, 0] = m_b_t(s.iloc[18, 0])  # Shares Outstanding
            s.iloc[19, 0] = m_b_t(s.iloc[19, 0])  # Implied Shares Outstanding
            s.iloc[20, 0] = m_b_t(s.iloc[20, 0])  # Float
            s.iloc[21, 0] = m_b_t(s.iloc[21, 0])  # % Held by Insiders
            s.iloc[22, 0] = m_b_t(s.iloc[22, 0])  # % Held by Institutions
            s.iloc[23, 0] = m_b_t(s.iloc[23, 0])  # Shares Short (Oct 14, 2021)
            s.iloc[24, 0] = float(s.iloc[24, 0])  # Short Ratio (Oct 14, 2021)
            s.iloc[25, 0] = m_b_t(s.iloc[25, 0])  # Short % of Float (Oct 14, 2021)
            s.iloc[26, 0] = m_b_t(s.iloc[26, 0])  # Short % of Shares Outstanding (Oct 14, 2021)
            s.iloc[27, 0] = m_b_t(s.iloc[27, 0])  # Shares Short (prior month Sep 14, 2021)
            s.iloc[28, 0] = float(s.iloc[28, 0])  # Forward Annual Dividend Rate
            s.iloc[29, 0] = m_b_t(s.iloc[29, 0])  # Forward Annual Dividend Yield
            s.iloc[30, 0] = float(s.iloc[30, 0])  # Trailing Annual Dividend Rate
            s.iloc[31, 0] = m_b_t(s.iloc[31, 0])  # Trailing Annual Dividend Yield
            s.iloc[32, 0] = float(s.iloc[32, 0])  # 5 Year Average Dividend Yield
            s.iloc[33, 0] = m_b_t(s.iloc[33, 0])  # Payout Ratio
            # s.iloc[34,0] = float(s.iloc[34,0]) # Dividend Date
            # s.iloc[35,0] = float(s.iloc[35,0]) # Ex-Dividend Date
            # s.iloc[36,0] = float(s.iloc[36,0]) # Last Split Factor
            # s.iloc[37,0] = float(s.iloc[37,0]) # Last Split Date
            # s.iloc[38,0] = float(s.iloc[38,0]) # Fiscal Year Ends
            # s.iloc[39,0] = float(s.iloc[39,0]) # Most Recent Quarter (mrq)
            s.iloc[40, 0] = m_b_t(s.iloc[40, 0])  # Profit Margin
            s.iloc[41, 0] = m_b_t(s.iloc[41, 0])  # Operating Margin (ttm)
            s.iloc[42, 0] = m_b_t(s.iloc[42, 0])  # Return on Assets (ttm)
            s.iloc[43, 0] = m_b_t(s.iloc[43, 0])  # Return on Equity (ttm)
            s.iloc[44, 0] = m_b_t(s.iloc[44, 0])  # Revenue (ttm)
            s.iloc[45, 0] = float(s.iloc[45, 0])  # Revenue Per Share (ttm)
            s.iloc[46, 0] = m_b_t(s.iloc[46, 0])  # Quarterly Revenue Growth (yoy)
            s.iloc[47, 0] = m_b_t(s.iloc[47, 0])  # Gross Profit (ttm)
            s.iloc[48, 0] = m_b_t(s.iloc[48, 0])  # EBITDA
            s.iloc[49, 0] = m_b_t(s.iloc[49, 0])  # Net Income Avi to Common (ttm)
            s.iloc[50, 0] = float(s.iloc[50, 0])  # Diluted EPS (ttm)
            s.iloc[51, 0] = m_b_t(s.iloc[51, 0])  # Quarterly Earnings Growth (yoy)
            s.iloc[52, 0] = m_b_t(s.iloc[52, 0])  # Total Cash (mrq)
            s.iloc[53, 0] = float(s.iloc[53, 0])  # Total Cash Per Share (mrq)
            s.iloc[54, 0] = m_b_t(s.iloc[54, 0])  # Total Debt (mrq)
            s.iloc[55, 0] = float(s.iloc[55, 0])  # Total Debt/Equity (mrq)
            s.iloc[56, 0] = float(s.iloc[56, 0])  # Current Ratio (mrq)
            s.iloc[57, 0] = float(s.iloc[57, 0])  # Book Value Per Share (mrq)
            s.iloc[58, 0] = m_b_t(s.iloc[58, 0])  # Operating Cash Flow (ttm)
            s.iloc[59, 0] = m_b_t(s.iloc[59, 0])  # Levered Free Cash Flow (ttm)

            s = s.rename(
                columns={self.ticker + ' Yahoo Statistics': self.ticker + ' Yahoo Statistics PreProcessing'})

        else:
            s = None

        return s


# -*- coding: utf-8 -*-
"""
Created on Wed Jul  6 18:02:49 2022

@author: RobWen
Version: 0.4.0
"""

# Packages
from StockHero.Ticker_Sources.NASDAQRequest import *
from StockHero.Ticker_Sources.MorningStarRequest import *
from StockHero.Ticker_Sources.YahooRequest import *
from StockHero.Ticker_Sources.GuruFocusRequest import *

    ##############
    ###        ###
    ###  Data  ###
    ###        ###
    ##############

class Ticker:
    
    def __init__(self, ticker):
        self.ticker = ticker
        self.__headers_standard = {"User-Agent" : "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0"}
        self.yahoo = YahooRequest(ticker = self.ticker, headers_standard = self.__headers_standard)
        self.nasdaq = NASDAQRequest(ticker = self.ticker, headers_standard = self.__headers_standard)
        self.morningstar = MorningStarRequest(ticker = self.ticker, headers_standard = self.__headers_standard)
        self.gurufocus = GuruFocusRequest(ticker = self.ticker, headers_standard = self.__headers_standard)
    
    def __repr__(self):
        return(self.ticker)
        
    def __str__(self):
        return(self.ticker)
        #return(self.ticker or '') # by None

###############################################################################
###############################################################################


# -*- coding: utf-8 -*-
"""
Created on Fri Aug 26 09:55:42 2022

@author: RobWen
Version: 0.4.0
"""

# Packages
import pandas as pd
import requests
from bs4 import BeautifulSoup
import numpy as np

# Header
from .TickerRequest import *

class MorningStarRequest(TickerRequest):
    def __init__(self, ticker, headers_standard):
        super().__init__(ticker, headers_standard)
        self.__headers_standard = headers_standard

    ################################
    ###                          ###
    ###  Morningstar Requests    ###
    ###                          ###
    ################################

    @property
    def quote(self):
        return self.__morningstar_quote_abfrage()

    @property
    def growth_rev(self):
        return self.__morningstar_quote_abfrage_growth_revenue()
    
    @property
    def growth_op_inc(self):
        return self.__morningstar_quote_abfrage_operating_income()
    
    @property
    def growth_net_inc(self):
        return self.__morningstar_quote_abfrage_net_income()
    
    @property
    def growth_eps(self):
        return self.__morningstar_quote_abfrage_growth_eps()
    
    ##########################
    ###                    ###
    ###  Morningstar Data  ###
    ###                    ###
    ##########################
    
    # Dummy Abfrage
    def __morningstar_quote_abfrage(self):
        
        ' Dummy Abfrage'
        if self.ticker is None or self.ticker == '':
            self.ticker = 'None'
            return None
        
        return self.__morningstar_quote_df()
    
    # Führt eine Abfrage durch um die Performance ID zu finden 
    def __morningstar_performance_id(self):
        url = "https://www.morningstar.co.uk/uk/funds/SecuritySearchResults.aspx"
        params = {'search': f'{self.ticker}'}
        
        data = requests.get(url, params=params, headers = self.__headers_standard)
        data = BeautifulSoup(data.content, 'html.parser')
        
        try:
            performance_id = data.find('td', {'class':'msDataText searchLink'})
            performance_id = performance_id.prettify().split()[4].split('=')[2].split(']')[0]
        except:
            return None
        
        return performance_id
    
    ### Morningstar Quote                                       ###
    ### e.g. https://www.morningstar.com/stocks/xnas/nvda/quote ###
    ### Rückgabe None implementiert und getestet                ###
    ### Ungültige Werte = NaN implementiert                     ###
    def __morningstar_quote_df(self):
        morningstar_performance_id = self.__morningstar_performance_id()
        
        if morningstar_performance_id is None:
            return None
        else:
            url = f'https://api-global.morningstar.com/sal-service/v1/stock/header/v2/data/{morningstar_performance_id}/securityInfo?showStarRating=&languageId=en&locale=en&clientId=MDC&benchmarkId=category&component=sal-components-quote&version=3.69.0'
            
            headers = {
                'ApiKey': 'lstzFDEOhfFNMLikKa0am9mgEKLBl49T',
            }
            
            r = requests.get(url, headers=headers)
            dictionary = r.json()
            
            priceEarnings = dictionary["priceEarnings"]
            priceBook = dictionary["priceBook"]
            priceSale = dictionary["priceSale"]
            forwardPE = dictionary["forwardPE"]
            forwardDivYield = dictionary["forwardDivYield"]
            
            url = f'https://api-global.morningstar.com/sal-service/v1/stock/keyStats/{morningstar_performance_id}?languageId=en&locale=en&clientId=MDC&benchmarkId=category&component=sal-components-quote&version=3.69.0'
            
            headers = {
                'ApiKey': 'lstzFDEOhfFNMLikKa0am9mgEKLBl49T',
            }
            
            r = requests.get(url, headers=headers)
            json = r.json()
            
            revenue3YearGrowth = json['revenue3YearGrowth']['stockValue']
            netIncome3YearGrowth = json['netIncome3YearGrowth']['stockValue']
            operatingMarginTTM = json['operatingMarginTTM']['stockValue']
            netMarginTTM = json['netMarginTTM']['stockValue']
            roaTTM = json['roaTTM']['stockValue']
            roeTTM = json['roeTTM']['stockValue']
            freeCashFlowTTM = json['freeCashFlow']['cashFlowTTM']
            
            try:
                priceEarnings = '{:.2f}'.format(float(priceEarnings))
                priceBook = '{:.2f}'.format(float(priceBook))
                priceSale = '{:.2f}'.format(float(priceSale))
                forwardPE = '{:.2f}'.format(float(forwardPE))
                forwardDivYield = float(forwardDivYield) * 100 # in %
                revenue3YearGrowth = '{:.2f}'.format(float(revenue3YearGrowth))
                netIncome3YearGrowth = '{:.2f}'.format(float(netIncome3YearGrowth))
                operatingMarginTTM = '{:.2f}'.format(float(operatingMarginTTM))
                netMarginTTM = '{:.2f}'.format(float(netMarginTTM))
                roaTTM = '{:.2f}'.format(float(roaTTM))
                roeTTM = '{:.2f}'.format(float(roeTTM))
                freeCashFlowTTM = '{:,.2f}'.format(float(freeCashFlowTTM)) # locale='en_US'
            except(TypeError):
                pass
            
            df_morningstar_quote = pd.DataFrame([priceEarnings, priceBook, priceSale, forwardPE, forwardDivYield
                               , revenue3YearGrowth, netIncome3YearGrowth, operatingMarginTTM, netMarginTTM, roaTTM, roeTTM
                               , freeCashFlowTTM]
                              , index =['Price/Earnings', 'Price/Book', 'Price/Sales', 'Consensus Forward P/E', 'Forward Div Yield %'
                                        , 'Rev 3-Yr Growth', 'Net Income 3-Yr Growth'
                                        , 'Operating Margin % TTM', 'Net Margin % TTM', 'ROA % TTM'
                                        , 'ROE % TTM', 'Current Free Cash Flow']
                              , columns =[self.ticker + ' Ratio'])
            
            df_morningstar_quote = df_morningstar_quote.fillna(value=np.nan) # None mit NaN ersetzen für df
        
        return df_morningstar_quote
    
    # Dummy Abfragen
    def __morningstar_quote_abfrage_growth_revenue(self):
        
        if self.ticker is None or self.ticker == '':
            self.ticker = 'None'
            return None
        
        return self.__morningstar_growth_revenue_df()
    
    def __morningstar_quote_abfrage_operating_income(self):
        
        if self.ticker is None or self.ticker == '':
            self.ticker = 'None'
            return None
        
        return self.__morningstar_operating_income_df()
    
    def __morningstar_quote_abfrage_net_income(self):
        
        if self.ticker is None or self.ticker == '':
            self.ticker = 'None'
            return None
        
        return self.__morningstar_net_income_df()
    
    def __morningstar_quote_abfrage_growth_eps(self):
        
        if self.ticker is None or self.ticker == '':
            self.ticker = 'None'
            return None
        
        return self.__morningstar_growth_eps_df()
    
    def __morningstar_growth_revenue_df(self):
        morningstar_performance_id = self.__morningstar_performance_id()
        
        if morningstar_performance_id is None:
            return None
        else:
            url = f'https://api-global.morningstar.com/sal-service/v1/stock/keyStats/growthTable/{morningstar_performance_id}?languageId=en&locale=en&clientId=undefined&component=sal-components-key-stats-growth-table&version=3.71.0'
    
            headers = {
                'ApiKey': 'lstzFDEOhfFNMLikKa0am9mgEKLBl49T',
            }
            
            r = requests.get(url, headers=headers)
            json = r.json()
            
            columns = []
            for i in range(len(json['dataList'])):
                columns.append(json['dataList'][i]['fiscalPeriodYearMonth'])
                
            liste_values = []
            for i in range(len(json['dataList'])):
                liste_values.append(list(json['dataList'][i]['revenuePer'].values()))
                
            array_table = np.array(liste_values).transpose()
            
            morningstar_growth_revenue_df = pd.DataFrame(array_table
                               , index =['Year over Year', '3-Year Average', '5-Year Average', '10-Year Average']
                               , columns = columns
                               )
            
            morningstar_growth_revenue_df = morningstar_growth_revenue_df.fillna(value=np.nan)
        
        return morningstar_growth_revenue_df
    
    def __morningstar_operating_income_df(self):
        morningstar_performance_id = self.__morningstar_performance_id()
        
        if morningstar_performance_id is None:
            return None
        else:
            url = f'https://api-global.morningstar.com/sal-service/v1/stock/keyStats/growthTable/{morningstar_performance_id}?languageId=en&locale=en&clientId=undefined&component=sal-components-key-stats-growth-table&version=3.71.0'
    
            headers = {
                'ApiKey': 'lstzFDEOhfFNMLikKa0am9mgEKLBl49T',
            }
            
            r = requests.get(url, headers=headers)
            json = r.json()
            
            columns = []
            for i in range(len(json['dataList'])):
                columns.append(json['dataList'][i]['fiscalPeriodYearMonth'])
                
            liste_values = []
            for i in range(len(json['dataList'])):
                liste_values.append(list(json['dataList'][i]['operatingIncome'].values()))
                
            array_table = np.array(liste_values).transpose()
            
            morningstar_operating_income_df = pd.DataFrame(array_table
                               , index =['Year over Year', '3-Year Average', '5-Year Average', '10-Year Average']
                               , columns = columns
                               )
            
            morningstar_operating_income_df = morningstar_operating_income_df.fillna(value=np.nan)
        
        return morningstar_operating_income_df
    
    def __morningstar_net_income_df(self):
        morningstar_performance_id = self.__morningstar_performance_id()
        
        if morningstar_performance_id is None:
            return None
        else:
            url = f'https://api-global.morningstar.com/sal-service/v1/stock/keyStats/growthTable/{morningstar_performance_id}?languageId=en&locale=en&clientId=undefined&component=sal-components-key-stats-growth-table&version=3.71.0'
    
            headers = {
                'ApiKey': 'lstzFDEOhfFNMLikKa0am9mgEKLBl49T',
            }
            
            r = requests.get(url, headers=headers)
            json = r.json()
            
            columns = []
            for i in range(len(json['dataList'])):
                columns.append(json['dataList'][i]['fiscalPeriodYearMonth'])
                
            liste_values = []
            for i in range(len(json['dataList'])):
                liste_values.append(list(json['dataList'][i]['netIncomePer'].values()))
                
            array_table = np.array(liste_values).transpose()
            
            morningstar_net_income_df = pd.DataFrame(array_table
                               , index =['Year over Year', '3-Year Average', '5-Year Average', '10-Year Average']
                               , columns = columns
                               )
            
            morningstar_net_income_df = morningstar_net_income_df.fillna(value=np.nan)
        
        return morningstar_net_income_df
    
    def __morningstar_growth_eps_df(self):
        morningstar_performance_id = self.__morningstar_performance_id()
        
        if morningstar_performance_id is None:
            return None
        else:
            url = f'https://api-global.morningstar.com/sal-service/v1/stock/keyStats/growthTable/{morningstar_performance_id}?languageId=en&locale=en&clientId=undefined&component=sal-components-key-stats-growth-table&version=3.71.0'
    
            headers = {
                'ApiKey': 'lstzFDEOhfFNMLikKa0am9mgEKLBl49T',
            }
            
            r = requests.get(url, headers=headers)
            json = r.json()
            
            columns = []
            for i in range(len(json['dataList'])):
                columns.append(json['dataList'][i]['fiscalPeriodYearMonth'])
                
            liste_values = []
            for i in range(len(json['dataList'])):
                liste_values.append(list(json['dataList'][i]['epsPer'].values()))
                
            array_table = np.array(liste_values).transpose()
            
            morningstar_growth_eps_df = pd.DataFrame(array_table
                               , index =['Year over Year', '3-Year Average', '5-Year Average', '10-Year Average']
                               , columns = columns
                               )
            
            morningstar_growth_eps_df = morningstar_growth_eps_df.fillna(value=np.nan)
        
        return morningstar_growth_eps_df


# -*- coding: utf-8 -*-
"""
Created on Fri Jul 22 09:17:42 2022

@author: RobWen
Version: 0.4.0
"""

class TickerRequest:
    def __init__(self, ticker, headers_standard):
        self.ticker = ticker
        #self.headers_standard = headers_standard

    def __repr__(self):
        return (self.ticker)

    def __str__(self):
        return (self.ticker)
        # return(self.Morningstar_Key_Ratios or '') # by None


# -*- coding: utf-8 -*-
"""
Created on Wed Jul  6 18:02:49 2022

@author: RobWen
Version: 0.3.3
"""
import pandas as pd
import requests
from pandas import json_normalize
from bs4 import BeautifulSoup

    #########################
    ###                   ###
    ###  Stock exchanges  ###
    ###     indicies      ###
    ###                   ###
    #########################
        
class StockExchange:

    def __init__(self, stockexchange):
        self.stockexchange = stockexchange
        self.__headers_standard = {"User-Agent" : "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0"}
        
    def __repr__(self):
        return(self.stockexchange)
        
    def __str__(self):
        return(self.stockexchange)
    
    #####################
    ###               ###
    ###    NASDAQ     ###
    ###               ###
    #####################
    
    @property
    def nasdaq(self):
        return self.__df_nasdaq()
    
    #####################
    ###               ###
    ###      CNN      ###
    ###               ###
    #####################
    
    @property
    def cnn_fear_and_greed(self):
        return self.__cnn_fear_and_greed_df()
    
    ##########################
    ###                    ###
    ###      Börsen        ###
    ###  Hamburg-Hannover  ###
    ###                    ###
    ##########################
    
    ''' Down since 23.06.2022
    
    @property
    def dax(self):
        return self.__boersenag_dax_df()
    
    @property
    def mdax(self):
        return self.__boersenag_mdax_df()
    
    @property
    def sdax(self):
        return self.__boersenag_sdax_df()
    
    @property
    def tecdax(self):
        return self.__boersenag_tecdax_df()
    
    @property
    def nisax(self):
        return self.__boersenag_nisax_df()
    
    @property
    def haspax(self):
        return self.__boersenag_haspax_df()
    
    @property
    def eurostoxx(self):
        return self.__boersenag_eurostoxx_df()
    
    @property
    def gcx(self):
        return self.__boersenag_gcx_df()
    
    @property
    def gevx(self):
        return self.__boersenag_gevx_df()
    
    @property
    def gergenx(self):
        return self.__boersenag_gergenx_df()
    
    @property
    def dow_jones(self):
        return self.__boersenag_dow_jones_df()
    
    @property
    def nasdaq_100(self):
        return self.__boersenag_nasdaq_100_df()
    
    '''
    
    #####################
    ###               ###
    ###    NASDAQ     ###
    ###               ###
    #####################
    
    ### NASDQ Stock Screener                                   ###
    ### https://www.nasdaq.com/market-activity/stocks/screener ###
    def __df_nasdaq(self):
        r = requests.get("https://api.nasdaq.com/api/screener/stocks?tableonly=true&limit=25&offset=0&download=true", headers=self.__headers_standard)
        
        json = r.json()
               
        json_data = json['data']['rows']
        df = json_normalize(json_data)
        json_headers = json['data']['headers']
        df_headers = json_normalize(json_headers)
        df_nasdaq_stockexchange = df.rename(columns=df_headers.loc[0])
        
        return df_nasdaq_stockexchange
    
    #####################
    ###               ###
    ###      CNN      ###
    ###               ###
    #####################
    
    ### CNN Fear and Greed Index                   ###
    ### https://money.cnn.com/data/fear-and-greed/ ###
    def __cnn_fear_and_greed_df(self):
        r = requests.get("https://production.dataviz.cnn.io/index/fearandgreed/graphdata", headers=self.__headers_standard)
        json = r.json()
        
        try:
            def fear_greed_f(fear_greed):
              wert = int(round(fear_greed))
              
              if wert < 0:
                  fear_greed_rating = 'Cant read values'
              elif wert < 25:
                  fear_greed_rating = 'Extreme Fear'
              elif wert < 46:
                  fear_greed_rating = 'Fear'
              elif wert < 55:
                  fear_greed_rating = 'Neutral'
              elif wert < 76:
                    fear_greed_rating = 'Greed'
              elif wert <= 100:
                    fear_greed_rating = 'Extreme Greed'
              else:
                    fear_greed_rating = 'Cant read values'
                    
              return fear_greed_rating
            
            df_cnn_fear_and_greed = pd.DataFrame(
                                [
                                [json['fear_and_greed']['score'], fear_greed_f(json['fear_and_greed']['score'])],
                                [json['fear_and_greed']['previous_close'], fear_greed_f(json['fear_and_greed']['previous_close'])],
                                [json['fear_and_greed']['previous_1_week'], fear_greed_f(json['fear_and_greed']['previous_1_week'])],
                                [json['fear_and_greed']['previous_1_month'], fear_greed_f(json['fear_and_greed']['previous_1_month'])],
                                [json['fear_and_greed']['previous_1_year'], fear_greed_f(json['fear_and_greed']['previous_1_year'])],
                                ]
                                
                                , index = ['Current', 'Previous close', '1 week ago', '1 month ago', '1 year ago']
                                , columns = ['Score', 'Rating'])
                       
            return df_cnn_fear_and_greed
        except:
            return None
        
    ##########################
    ###                    ###
    ###      Börsen        ###
    ###  Hamburg-Hannover  ###
    ###                    ###
    ##########################
    
    ### DAX Performance-Index                           ###
    ### 40 Werte                                        ###
    ### https://www.boersenag.de/Index/DE0008469008/DAX ###

    def __boersenag_dax_df(self):
        url = 'https://www.boersenag.de/Index/DE0008469008/DAX'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        number = page.find('div', {'class':'pager'}).text.split()[-2]               # String
        
        url = f'https://www.boersenag.de/Index/DE0008469008/DAX?p=1&pager=780&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        table = page.find('table', {'class':'table table-striped table-prices'})
        
        try:
            page = page.find_all('a')
            columns = table.find_all('th')
            
            data = []
            
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
            
            index_array = list(range(1,int(number)+1))
            
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
            
            __df_boersenag_dax = pd.DataFrame(
                                data,
                                index = index_array,
                                columns = columns_array
                                )
            
            return __df_boersenag_dax
        except:
            return None
        
    ### MDAX Performance-Index                            ###
    ### 50 Werte                                          ###
    ### https://www.boersenag.de/Index/DE0008467416/MDAX  ###
    
    def __boersenag_mdax_df(self):
        url = 'https://www.boersenag.de/Index/DE0008467416/MDAX'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        number = page.find('div', {'class':'pager'}).text.split()[-2]                           # String
    
        url = f'https://www.boersenag.de/Index/DE0008467416/MDAX?p=1&pager=811&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        table = page.find('table', {'class':'table table-striped table-prices'})
        
        try:
            page = page.find_all('a')
            columns = table.find_all('th')
        
            data = []
        
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
        
            index_array = list(range(1,int(number)+1))
        
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
        
            __df_boersenag_mdax = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )
    
            return __df_boersenag_mdax
        except:
            return None
        
    ### SDAX Performance-Index                            ###
    ### 70 Werte                                          ###
    ### https://www.boersenag.de/Index/DE0009653386/SDAX  ###
    
    def __boersenag_sdax_df(self):
        url = 'https://www.boersenag.de/Index/DE0009653386/SDAX'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        number = page.find('div', {'class':'pager'}).text.split()[-2]
    
        url = f'https://www.boersenag.de/Index/DE0009653386/SDAX?p=1&pager=842&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        table = page.find('table', {'class':'table table-striped table-prices'})
        try:
            page = page.find_all('a')
            columns = table.find_all('th')
        
            data = []
        
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
        
            index_array = list(range(1,int(number)+1))
        
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
        
            __df_boersenag_sdax = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )

            return __df_boersenag_sdax
        except:
            return None
    
    ### TecDAX Performance-Index                            ###
    ### 30 Werte                                            ###
    ### https://www.boersenag.de/Index/DE0007203275/TecDAX  ###
    
    def __boersenag_tecdax_df(self):
        url = 'https://www.boersenag.de/Index/DE0007203275/TecDAX'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        number = page.find('div', {'class':'pager'}).text.split()[-2]
    
        url = f'https://www.boersenag.de/Index/DE0007203275/TecDAX?p=1&pager=815&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        table = page.find('table', {'class':'table table-striped table-prices'})
        try:
            page = page.find_all('a')
            columns = table.find_all('th')
        
            data = []
        
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
        
            index_array = list(range(1,int(number)+1))
        
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
        
            __df_boersenag_tecdax = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )
    
            return __df_boersenag_tecdax
        except:
            return None
    
    ### NISAX 20 Index (Net Return) (EUR)                   ###
    ### 20 Werte                                            ###
    ### https://www.boersenag.de/Index/DE000A2BL7T2/Nisax20 ###
    
    def __boersenag_nisax_df(self):
        url = 'https://www.boersenag.de/Index/DE000A2BL7T2/Nisax20'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        number = page.find('div', {'class':'pager'}).text.split()[-2]
        
        url = f'https://www.boersenag.de/Index/DE000A2BL7T2/Nisax20?p=1&pager=4891&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        try:
            table = page.find('table', {'class':'table table-striped table-prices'})
            
            page = page.find_all('a')
            columns = table.find_all('th')
            
            data = []
            
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
            
            index_array = list(range(1,int(number)+1))
            
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
            
            __df_boersenag_nisax = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )
        
            return __df_boersenag_nisax
        except:
            return None
    
    ### Haspax Index (Performance) (EUR)                   ###
    ### 22 Werte (01.12.2021)                              ###
    ### https://www.boersenag.de/Index/DE0008468810/Haspax ###
    
    def __boersenag_haspax_df(self):
        url = 'https://www.boersenag.de/Index/DE0008468810/Haspax'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        number = page.find('div', {'class':'pager'}).text.split()[-2]
    
        url = f'https://www.boersenag.de/Index/DE0008468810/Haspax?p=1&pager=1093&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        try:
            table = page.find('table', {'class':'table table-striped table-prices'})
        
            page = page.find_all('a')
            columns = table.find_all('th')
        
            data = []
        
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
        
            index_array = list(range(1,int(number)+1))
        
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
        
            __df_boersenag_haspax = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )
    
            return __df_boersenag_haspax
        except:
            return None
        
    ### EURO STOXX 50 Index (Price) (EUR)                       ###
    ### 50 Werte                                                ###
    ### https://www.boersenag.de/Index/EU0009658145/EuroStoxx50 ###
    
    def __boersenag_eurostoxx_df(self):
        url = 'https://www.boersenag.de/Index/EU0009658145/EuroStoxx50'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        number = page.find('div', {'class':'pager'}).text.split()[-2]
    
        url = f'https://www.boersenag.de/Index/EU0009658145/EuroStoxx50?p=1&pager=1293&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        try:
            table = page.find('table', {'class':'table table-striped table-prices'})
        
            page = page.find_all('a')
            columns = table.find_all('th')
        
            data = []
        
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
        
            index_array = list(range(1,int(number)+1))
        
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
        
            __df_boersenag_eurostoxx = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )

            return __df_boersenag_eurostoxx
        except:
            return None
    
    ### GCX Global Challenges Performance-Index         ###
    ### 50 Werte                                        ###
    ### https://www.boersenag.de/Index/DE000A0MEN25/GCX ###
    
    def __boersenag_gcx_df(self):
        url = 'https://www.boersenag.de/Index/DE000A0MEN25/GCX'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        number = page.find('div', {'class':'pager'}).text.split()[-2]
    
        url = f'https://www.boersenag.de/Index/DE000A0MEN25/GCX?p=1&pager=818&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
    
        try:
            table = page.find('table', {'class':'table table-striped table-prices'})
        
            page = page.find_all('a')
            columns = table.find_all('th')
        
            data = []
        
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
        
            index_array = list(range(1,int(number)+1))
        
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
        
            __df_boersenag_gcx = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )

            return __df_boersenag_gcx
        except:
            return None

    ### Global Ethical Values Index (Total Return) (EUR)  ###
    ### 609 Werte (variable)                              ###
    ### https://www.boersenag.de/Index/DE000SL0EBW8/GEVX  ###
    
    def __boersenag_gevx_df(self):
        url = 'https://www.boersenag.de/Index/DE000SL0EBW8/GEVX'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        number_gesamt = page.find('div', {'class':'pager'}).text.split()[-2]           # String
        
        data = []
        
        try:
        
            for seite in range(1, int(int(number_gesamt)/100)+2):
             
                number = '100'
                
                url = f'https://www.boersenag.de/Index/DE000SL0EBW8/GEVX?p={seite}&pager=5345&limit={number}'
                page = requests.get(url)
                page = BeautifulSoup(page.content, 'html.parser')
                
                table = page.find('table', {'class':'table table-striped table-prices'})
                
                page = page.find_all('a')
                columns = table.find_all('th')
                
                if seite > int(int(number_gesamt)/100):
                    number = str(int(number_gesamt) % 100)
                
                for i in range(int(number)):
                    data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
                
            
            index_array = list(range(1,int(number_gesamt)+1))
            
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
            
            __df_boersenag_gevx = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )
                
            return __df_boersenag_gevx
        except:
            return None

    ### German Gender Index (Total Return) (EUR)                       ###
    ### 50 Werte                                                       ###
    ### https://www.boersenag.de/Index/DE000SLA0QF8/GermanGenderIndex  ###
    
    def __boersenag_gergenx_df(self):
        url = 'https://www.boersenag.de/Index/DE000SLA0QF8/GermanGenderIndex'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        number = page.find('div', {'class':'pager'}).text.split()[-2]
        
        url = f'https://www.boersenag.de/Index/DE000SLA0QF8/GermanGenderIndex?p=1&pager=1308&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        try:
            table = page.find('table', {'class':'table table-striped table-prices'})
            
            page = page.find_all('a')
            columns = table.find_all('th')
            
            data = []
            
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
            
            index_array = list(range(1,int(number)+1))
            
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
            
            __df_boersenag_gergenx = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )
        
            return __df_boersenag_gergenx
        except:
            return None


    ### Dow Jones Industrial Average Index (Price) (USD)       ###
    ### 30 Werte (fix) - Fehler hier nur 29 Werte (01.12.2021) ###
    ### https://www.boersenag.de/Index/US2605661048/DowJones   ###
    
    def __boersenag_dow_jones_df(self):
        url = 'https://www.boersenag.de/Index/US2605661048/DowJones'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        number = page.find('div', {'class':'pager'}).text.split()[-2]

        url = f'https://www.boersenag.de/Index/US2605661048/DowJones?p=1&pager=1323&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        try:
            table = page.find('table', {'class':'table table-striped table-prices'})
            
            page = page.find_all('a')
            columns = table.find_all('th')
            
            data = []
            
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
            
            index_array = list(range(1,int(number)+1))
            
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
            
            __df_boersenag_dow_jones = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )
    
            return __df_boersenag_dow_jones
        except:
            return None

    ### Nasdaq-100 Index                                        ###
    ### 100 Werte (fix) - Fehler hier nur 86 Werte (01.12.2021) ###
    ### https://www.boersenag.de/Index/US6311011026/Nasdaq      ###
    
    def __boersenag_nasdaq_100_df(self):
        url = 'https://www.boersenag.de/Index/US6311011026/Nasdaq'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        number = page.find('div', {'class':'pager'}).text.split()[-2]
        
        url = f'https://www.boersenag.de/Index/US6311011026/Nasdaq?p=1&pager=1338&limit={number}'
        page = requests.get(url)
        page = BeautifulSoup(page.content, 'html.parser')
        
        try:
            table = page.find('table', {'class':'table table-striped table-prices'})
            
            page = page.find_all('a')
            columns = table.find_all('th')
            
            data = []
            
            for i in range(int(number)):
                data.append([page[i].prettify().split('\n')[2].strip().replace('amp;',''), page[i].prettify().split('\n')[-2].strip()])
            
            index_array = list(range(1,int(number)+1))
            
            columns_array = [
                            columns[0].prettify().split()[1], 
                            columns[0].prettify().split()[3]
                            ]
            
            __df_boersenag_nasdaq_100 = pd.DataFrame(
                            data,
                            index = index_array,
                            columns = columns_array
                            )

            return __df_boersenag_nasdaq_100
        except:
            return None

###############################################################################
###############################################################################
    


# -*- coding: utf-8 -*-
"""
Created on Wed Jul  6 18:02:49 2022

@author: RobWen
Version: 0.3.3
"""
import pandas as pd
import numpy as np
from StockHero import Ticker

###############################################################################
###############################################################################


    ############################
    ###                      ###
    ###  Screener Summary    ###
    ###  under construction  ###
    ###                      ###
    ############################
    
# Basis
class StockValuation:

    def __init__(self, ticker):
        self.ticker = ticker
        
    def __repr__(self):
        return(self.ticker)
        
    def __str__(self):
        return(self.ticker)
    
    @property
    def valuation(self):
        return self.__df_valuation()
    
    def __df_valuation(self):
        
        arrays = [
            np.array([  "Fundamental"
                      , "Valuation Ratio"
                      , "Valuation Ratio"
                      , "Valuation Ratio"
                      , "Profitability"
                      , "Price"
                      , "Dividends"
                      , "Income Statement"
                      , "Income Statement"
                      , "Balance Sheet"
                      , "Cashflow Statement"
                      , "Technical Indicator"
                      ]),
            
            np.array([  "5-Year EBITDA Growth Rate"
                      , "Debt-to-EBITDA"
                      , "PE-Ratio (TTM)"
                      , "PE-Ratio (Forward)"
                      , "FCF Margin %"
                      , "Beta"
                      , "3-Year Dividend Growth Rate"
                      , "EBIT"
                      , "EBITDA"
                      , "Long-Term Debt"
                      , "Free Cash Flow"
                      , "50-Day SMA"
                      ])]
        
        # Yahoo Finanance # Morningstar # NASDAQ
        array_table = [
                        # Fundamental / 5-Year EBITDA Growth Rate
                        [np.nan
                       , np.nan
                       , np.nan]                              
                       # Valuation Ratio / Debt-to-EBITDA
                       , [self.debt_to_ebitda_yahoo()
                       , np.nan
                       , np.nan]
                       # Valuation Ratio / PE-Ratio (TTM)
                       , [Ticker(self.ticker).yahoo_statistics_p.iloc[2,0]
                       , np.nan
                       , np.nan]
                       # Valuation Ratio / PE-Ratio (Forward)
                       , [Ticker(self.ticker).yahoo_statistics_p.iloc[3,0]
                       , np.nan
                       , np.nan]
                       # Profitability / FCF Margin %
                       , [np.nan
                       , np.nan
                       , np.nan]
                       # Price / Beta
                       , [np.nan
                       , np.nan
                       , np.nan]
                       # Dividends / 3-Year Dividend Growth Rate
                       , [np.nan
                       , np.nan
                       , np.nan]
                       # Income Statement / EBIT
                       , [np.nan
                       , np.nan
                       , np.nan]
                       # Income Statement / EBITDA
                       , [Ticker(self.ticker).yahoo_statistics_p.iloc[48,0]
                       , np.nan
                       , np.nan]
                       # Balance Sheet / Long-Term Debt
                       , [np.nan
                       , np.nan
                       , np.nan]
                       # Cashflow Statement / Free Cash Flow
                       , [np.nan
                       , np.nan
                       , np.nan]
                       # Technical Indicator / 50-Day SMA
                       , [np.nan
                       , np.nan
                       , np.nan]
                       ]
            
        self.valuation_df = pd.DataFrame(array_table , index=arrays, columns = ['Yahoo Finance', 'Morningstar', 'NASDAQ'])
        
        return self.valuation_df
    
    def debt_to_ebitda_yahoo(self):
        
        ebitda = Ticker(self.ticker).yahoo_statistics_p.iloc[48,0]
        debt = Ticker(self.ticker).yahoo_statistics_p.iloc[54,0]
        
        debt_to_ebitda_yahoo_float = debt/ebitda
        
        return debt_to_ebitda_yahoo_float
    
###############################################################################
###############################################################################
    
    
    
    
    
    
    
    
    
    